package cg;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.image.BufferedImage;

import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
import javax.swing.plaf.metal.MetalLookAndFeel;

/**
 * For bringing objects onto the screen.
 */
public class Canvas {
	
	private static class Component extends JComponent {
		
		private BufferedImage buff;
		
		public Component(BufferedImage buff) {
			this.buff = buff;
		}
		
		public void paint(Graphics g) {
			if (this.buff == null) {
				return;
			}
			int width = this.buff.getWidth(this);
			int height = this.buff.getHeight(this);
			g.drawImage(	this.buff, 0, 0, width, height, this);
		}
		
	}
	
	private int width;
	private int height;
	
	private BufferedImage buff;
	private Component component;
	private JFrame mainFrame;	

	/**
	 * Creates and displays a canvas for drawing.
	 * @param width
	 * @param height
	 */
	public Canvas(int width, int height) {
		this.width = width;
		this.height = height;
		
		this.buff = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
		this.component = new Component(buff);

		try {
			UIManager.setLookAndFeel(new MetalLookAndFeel());
		} catch (UnsupportedLookAndFeelException e) {
			System.err.println("error while setting look-and-feel");
		}		
		this.mainFrame = new JFrame();
		
        javax.swing.SwingUtilities.invokeLater(new Runnable() {
            public void run() {
        		mainFrame.addWindowListener(new WindowAdapter() {
        			public void windowClosing(WindowEvent e) {
        				System.exit(0);
        			}
        		});
        		mainFrame.getContentPane().setLayout(new BorderLayout());
        		mainFrame.getContentPane().add(component);
            	mainFrame.setSize(Canvas.this.width, Canvas.this.height);
               	mainFrame.setVisible(true);                
            	int currentWidth = component.getSize().width;
            	int currentHeight = component.getSize().height;
            	mainFrame.setSize(2 * Canvas.this.width - currentWidth , 2 * Canvas.this.height - currentHeight + 4);
            	mainFrame.validate();
            }
        });

	}
	
	/**
	 * The width.
	 * @return The width.
	 */
	public int getWidth() {
		return this.width;
	}
	
	/**
	 * The height.
	 * @return The height.
	 */
	public int getHeight() {
		return this.height;
	}
		
	/**
	 * Causes a repaint of the canvas.
	 */
	public void update() {
		this.component.repaint();
	}
	
	/**
	 * Fills the complete canvas with a given color.
	 * @param color
	 */
	public void fill(Color color) {
		Graphics g = this.buff.getGraphics();
		g.setColor(color);
		g.fillRect(0, 0, this.width, this.height);
	}
	
	/**
	 * Draws a pixel with given screen-coordinates and given color.
	 * @param x
	 * @param y
	 * @param color
	 */
	public void drawPixel(int x, int y, Color color) {
		Graphics g = this.buff.getGraphics();
		g.setColor(color);
		g.drawLine(x, y, x, y);		
	}
	
	/**
	 * Draws a line segment with given screen-coordinates of the two endpoints and with given color.
	 * @param x1
	 * @param y1
	 * @param x2
	 * @param y2
	 * @param color
	 */
	public void drawLine(int x1, int y1, int x2, int y2, Color color) {
		Graphics g = this.buff.getGraphics();
		g.setColor(color);
		g.drawLine(x1, y1, x2, y2);		
	}
}
